import ij.ImagePlus;
import ij.LookUpTable;
import ij.gui.GenericDialog;
import ij.measure.Calibration;
import ij.plugin.PlugIn;
import ij.process.ImageProcessor;
import imagescience.image.Aspects;
import imagescience.image.ColorImage;
import imagescience.image.Dimensions;
import imagescience.image.Image;
import imagescience.transform.Scale;
import imagescience.utility.I5DResource;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.image.ColorModel;

public class TJ_Scale implements PlugIn {

    private static String xfactor = "1.0";
    private static String yfactor = "1.0";
    private static String zfactor = "1.0";
//     private static String tfactor = "1.0";
//     private static String cfactor = "1.0";

    private static final String[] schemes = {
	"nearest neighbor",
	"linear",
	"cubic convolution",
	"cubic B-spline",
	"cubic O-MOMS",
	"quintic B-spline"
    };
    private static int scheme = 1;

    public void run(String arg) {

	if (!TJ.libcheck()) return;
	final ImagePlus imp = TJ.imageplus();
	if (imp == null) return;

	TJ.debug(TJ.name()+" "+TJ.version()+": Scale");

	GenericDialog gd = new GenericDialog(TJ.name()+": Scale");
	gd.addStringField("x-factor for scaling:",xfactor);
	gd.addStringField("y-factor for scaling:",yfactor);
	gd.addStringField("z-factor for scaling:",zfactor);
// 	gd.addStringField("t-factor for scaling:",tfactor);
// 	gd.addStringField("c-factor for scaling:",cfactor);
	gd.addPanel(new Panel(),GridBagConstraints.WEST,new Insets(0,0,0,0));
	gd.addChoice("Interpolation scheme:",schemes,schemes[scheme]);
	gd.showDialog();

	if (gd.wasCanceled()) return;

	xfactor = gd.getNextString();
	yfactor = gd.getNextString();
	zfactor = gd.getNextString();
// 	tfactor = gd.getNextString();
// 	cfactor = gd.getNextString();
	scheme = gd.getNextChoiceIndex();

	(new TJScale()).run(imp,xfactor,yfactor,zfactor,scheme);
    }

}

class TJScale { // Separate class to make the plugin class independent of imagescience.jar

    void run(final ImagePlus imp,
	     final String xfactor,
	     final String yfactor,
	     final String zfactor,
	     final int scheme) {

	try {
	    final Image img = Image.wrap(imp);
	    final Scale scaler = new Scale();
	    scaler.debug(TJ_Options.debug);
	    double xf=1, yf=1, zf=1, tf=1, cf=1;
	    try { xf = Double.parseDouble(xfactor); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid x-factor for scaling"); }
	    try { yf = Double.parseDouble(yfactor); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid y-factor for scaling"); }
	    try { zf = Double.parseDouble(zfactor); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid z-factor for scaling"); }
// 	    try { tf = Double.parseDouble(tfactor); }
// 	    catch (Exception e) { throw new IllegalArgumentException("Invalid t-factor for scaling"); }
// 	    try { cf = Double.parseDouble(cfactor); }
// 	    catch (Exception e) { throw new IllegalArgumentException("Invalid c-factor for scaling"); }
	    int ischeme = Scale.NEAREST;
	    switch (scheme) {
	    case 0: ischeme = Scale.NEAREST; break;
	    case 1: ischeme = Scale.LINEAR; break;
	    case 2: ischeme = Scale.CUBIC; break;
	    case 3: ischeme = Scale.BSPLINE3; break;
	    case 4: ischeme = Scale.OMOMS3; break;
	    case 5: ischeme = Scale.BSPLINE5; break;
	    }
	    final Image newimg = scaler.run(img,xf,yf,zf,tf,cf,ischeme);
	    ImagePlus newimp = newimg.imageplus();
	    final Aspects newasps = newimg.aspects();
	    final Calibration cal = imp.getCalibration().copy();
	    cal.pixelWidth = newasps.x;
	    cal.pixelHeight = newasps.y;
	    cal.pixelDepth = newasps.z;
	    cal.frameInterval = newasps.t;
	    newimp.setCalibration(cal);
	    final ImageProcessor ip = imp.getProcessor();
	    final ImageProcessor newip = newimp.getProcessor();
	    final ColorModel cm = ip.getColorModel();
	    newip.setColorModel(cm);
	    newimp.getStack().setColorModel(cm);
	    final double[] minmax = newimg.extrema();
	    if (TJ_Options.adopt) newip.setMinAndMax(ip.getMin(),ip.getMax());
	    else newip.setMinAndMax(minmax[0],minmax[1]);
	    boolean i5dinstalled = false;
	    try { Class.forName("i5d.Image5D"); i5dinstalled = true; } catch (Exception e) { }
	    if (i5dinstalled && !(newimg instanceof ColorImage)) {
		final Dimensions newdims = newimg.dimensions();
		final Dimensions indims = img.dimensions();
		if (newdims.t > 1 || newdims.s > 1) {
		    newimp = I5DResource.convert(newimp,true);
		    if (I5DResource.instance(imp))
			if (newdims.s == indims.s) {
			    I5DResource.transfer(imp,newimp);
			    if (!TJ_Options.adopt)
				I5DResource.minmax(newimp,minmax[0],minmax[1]);
			}
			else { I5DResource.nodensity(newimp); I5DResource.nocolor(newimp); }
		} else if (indims.t > 1 || indims.s > 1) {
		    if (I5DResource.instance(imp)) {
			cal.disableDensityCalibration();
			newimp.setCalibration(cal);
			final ColorModel gcm = LookUpTable.createGrayscaleColorModel(false);
			newimp.getProcessor().setColorModel(gcm);
			newimp.getStack().setColorModel(gcm);
		    }
		}
	    }
	    newimp.changes = TJ_Options.asksave;
	    newimp.show();

	} catch (OutOfMemoryError e) {
	    TJ.error("Not enough memory for this operation");

	} catch (UnknownError e) {
	    TJ.error("Could not create output image for some reason.\nPossibly there is not enough free memory");

	} catch (IllegalArgumentException e) {
	    TJ.error(e.getMessage());

	} catch (Throwable e) {
	    TJ.error("An unidentified error occurred while running the plugin");

	}
    }

}
